/*************************************************************************************
**
** Description         : VFD Library (Vacuum Fluorescent Display Library) Test Program.
**                       This software used the "VFD Library" and a Atmel ATmega32
**                       Device for the Noritake CU20025-TW200A VFD Display
**                       This is a 5x7 Dot Character VFD Modul with 2 lines and 
**                       20 character per line.
**
** Compiler            : WinAVR 20081205
** Hardware            : AVR-Ctrl
** Chip type           : ATMega32
** Clock frequency     : 12MHz
** Autor               : Michael Odenwald
** License             : modified BSD license
** Date                : 31.12.2008
**
**************************************************************************************/
#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>

#include "vfd.h"

/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 12000000UL
#endif

/* communication speed with the VFD diaplay is 4800 baud */
#define UART_BAUD_RATE      4800

/* macro for UBRR calculation (check the Atmel datasheet for this) */
#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)

/* the low level function, to send one character to the VFD display */
int uart_putc(uint8_t c)
{
    while (!(UCSRA & (1<<UDRE)))  /* wait until sending is posible */
    {
	}
    UDR = c;                      /* send char */
    return 0;
}

/*
** This is the main function on the test program
**
** The program demonstrate some VFD library function
**
*/
int main(void)
{
	uint8_t a,x;

    /* USART-Init for ATMega32 */
    UCSRB |= ( 1 << TXEN );					// switch UART TX on
    UCSRC |= ( 1 << URSEL )|( 3<<UCSZ0 );	// use asynchron 8N1 for communication

  	/* set up the UBRR for the given baudrate */
    UBRRH = (uint8_t)(UART_UBRR_CALC( UART_BAUD_RATE, F_CPU ) >> 8 );
    UBRRL = (uint8_t) UART_UBRR_CALC( UART_BAUD_RATE, F_CPU );

	vfd_init(uart_putc);
	vfd_clrscr();
	vfd_puts_P("Step 1:");
	vfd_cursor_on_blinking();
	vfd_cursor_on_char();
	vfd_putc('*');
	vfd_gotoxy(1,2);
	vfd_puts_P("AVR-Ctrl with Atmel");
	_delay_ms(5000);
		
	vfd_clrscr();
	vfd_puts_P("Step 2:");
	vfd_gotoxy(1,2);
	vfd_version();
	_delay_ms(5000);
	
	vfd_clrscr();
	vfd_puts_P("Step 3:");
	vfd_katafont();
	vfd_cursor_off();
	for (a=0; a<=10; a++)
	{
		vfd_gotoxy(1,2);
		vfd_putc(0xF0);
		_delay_ms(200);
		vfd_gotoxy(1,2);
		vfd_putc(0xF1);
		_delay_ms(200);
		vfd_gotoxy(1,2);
		vfd_putc(0xF2);
		_delay_ms(200);
		vfd_gotoxy(1,2);
		vfd_putc(0xF3);
		_delay_ms(200);
		vfd_gotoxy(1,2);
		vfd_putc(0xF8);
		_delay_ms(200);
	}		
	_delay_ms(1000);

	vfd_clrscr();
	vfd_puts_P("Step 4:");
	vfd_intfont();
	for (a=0x21; a<=0x50; a++)
	{
		for (x=1; x<=20; x++)
		{
			vfd_gotoxy(x, 2);
			vfd_putc(a);
			_delay_ms(50);
		}
	}
	_delay_ms(5000);	

	vfd_clrscr();
	vfd_puts_P("Step 5:");
	vfd_cursor_off();
	vfd_gotoxy(1,2);
	for (a=0; a<=100; a++)
	{
		vfd_putc('|');
		_delay_ms(100);
		vfd_bs(VFD_BS_DESTRUCTIVE);	
		vfd_putc('/');
		_delay_ms(100);
		vfd_bs(VFD_BS_DESTRUCTIVE);	
		vfd_putc('-');
		_delay_ms(100);
		vfd_bs(VFD_BS_DESTRUCTIVE);	
		vfd_putc('\\');
		_delay_ms(100);
		vfd_bs(VFD_BS_DESTRUCTIVE);	
	}

	vfd_clrscr();
	vfd_puts_P("Demo Ende.");
	
	while(1)
	;
}
